/// <reference types="pouchdb-core" />
/// <reference types="node" />
/// <reference types="node" />
/// <reference types="mocha" />
/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
/// <reference types="pouchdb-find" />
/// <reference types="pouchdb-mapreduce" />
/// <reference types="pouchdb-replication" />
import { BigNumber } from 'ethers';
import { AsyncUtils } from '@tornado/sdk-utils';
export interface TornadoInstance {
    network: number;
    symbol: string;
    decimals: number;
    denomination: number;
    deployBlock: number;
    address: string;
}
export interface ClassicInstance extends TornadoInstance {
    anonymityMiningEnabled: boolean;
}
export interface TokenData {
    network: number;
    decimals: number;
    address: string;
}
export declare namespace Keys {
    interface InstanceLookup {
        network: string;
        token: string;
        denomination: string;
    }
}
export interface RelayerProperties {
    address: string;
    version: string;
    serviceFeePercent: number;
    miningFeePercent: number;
    status: string;
    chainId: number;
    prices: Map<string, BigNumber>;
}
export declare namespace Options {
    interface Cache {
        adapter?: string;
        dirPath?: string;
        persistent?: boolean;
    }
}
export declare namespace Files {
    type PathGetter = (relative: string) => string;
    function parentPath(filepath: string): string;
    function stripExtensions(filepath: string): string;
    const getModulesPath: (relative?: string, prefix?: string) => string;
    const getResourcePath: (relative?: string, prefix?: string) => string;
    const getCachePath: (relative?: string, prefix?: string) => string;
    const cacheDirExists: (prefix?: string) => boolean;
    const makeCacheDir: (prefix?: string) => void;
    const loadRaw: (relative: string) => Promise<Buffer>;
    const loadRawSync: (relative: string) => Buffer;
    const writeRaw: (relative: string, data: string | NodeJS.ArrayBufferView, flag: string) => Promise<void>;
    const writeRawSync: (relative: string, data: string | NodeJS.ArrayBufferView, flag: string) => void;
    function gzipSync(fileOrDirPath: string, archivePath: string): void;
    function gunzipSync(archivePath: string, extractPath: string): void;
    function wipeCache(prompt?: boolean, dirPath?: string): Promise<void>;
}
export declare namespace Json {
    function load(relativePath: string, encoding?: BufferEncoding, pathGetter?: Files.PathGetter): Promise<any>;
    function loadSync(relativePath: string, encoding?: BufferEncoding, pathGetter?: Files.PathGetter): any;
    function toMap<V>(jsonData: any): Map<string, V>;
    function loadMap<V>(relativePath: string, encoding?: BufferEncoding): Promise<Map<string, V>>;
    function getError(...values: any[]): Error;
    function throwError(...values: any[]): void;
    function getValue(jsonObj: any, keys: any[]): any;
}
export declare namespace Onchain {
    function getClassicInstanceData(network: string, token: string, denomination: string): Promise<ClassicInstance>;
    function getClassicInstanceDataSync(network: string, token: string, denomination: string): ClassicInstance;
    function getInstanceLookupKeys(instanceAddress: string): Promise<Keys.InstanceLookup>;
    function getInstanceLookupKeysSync(instanceAddress: string): Keys.InstanceLookup;
    function getPathstringBasedContent<T>(filepath: string, paths: Array<{
        network?: string;
        token?: string;
        denomination?: string;
    }>): Promise<Array<T>>;
    function getPathstringBasedContentSync<T>(filepath: string, paths: Array<{
        network?: string;
        token?: string;
        denomination?: string;
    }>): Array<T>;
    function getNetworkSymbol(networkId: string): Promise<string>;
    function getNetworkSymbolSync(networkId: string): string;
    function getInstanceAddresses(paths: Array<{
        network: string;
        token: string;
        denomination: string;
    }>): Promise<Array<string>>;
    function getInstanceAddressesSync(paths: Array<{
        network: string;
        token: string;
        denomination: string;
    }>): Array<string>;
    function getInstanceAddress(network: string, token: string, denomination: string): Promise<string>;
    function getInstanceAddressSync(network: string, token: string, denomination: string): string;
    function getRegistryAddress(network?: string): Promise<string>;
    function getRegistryAddressSync(network?: string): string;
    function getRegistryDeployBlockNum(network?: string): Promise<number>;
    function getRegistryDeployBlockNumSync(network?: string): number;
    function getInstanceDeployBlockNums(paths: Array<{
        network: string;
        token: string;
        denomination: string;
    }>): Promise<Array<number>>;
    function getInstanceDeployBlockNumsSync(paths: Array<{
        network: string;
        token: string;
        denomination: string;
    }>): Array<number>;
    function getInstanceDeployBlockNum(network: string, token: string, denomination: string): Promise<number>;
    function getInstanceDeployBlockNumSync(network: string, token: string, denomination: string): number;
    function getProxyAddress(network: string): Promise<string>;
    function getMulticallAddress(network: string): Promise<string>;
    function getMulticall3Address(network: string): Promise<string>;
    function getProxyAddressSync(network: string): string;
    function getMulticallAddressSync(network: string): string;
    function getMulticall3AddressSync(network: string): string;
    function getTokenAddress(network: string, token: string): Promise<string>;
    function getTokenAddressSync(network: string, token: string): string;
    function getTokenDecimals(network: string, token: string): Promise<number>;
    function getTokenDecimalsSync(network: string, token: string): number;
}
export declare namespace Offchain {
    function getUncensoredRpcURL(network: string, name?: string): Promise<string>;
    function getUncensoredRpcURLSync(network: string, name?: string): string;
    function getClassicSubgraphURL(network: string): Promise<string>;
    function getClassicSubgraphURLSync(network: string): string;
}
export declare namespace Constants {
    const MERKLE_TREE_HEIGHT = 20;
}
export declare namespace Docs {
    class Base {
        _id: string;
        _rev?: string;
        constructor(id: string);
    }
    class Deposit extends Base {
        blockNumber: number;
        leafIndex: number;
        commitment: string;
        transactionHash: string;
        timestamp: string;
        constructor(obj: any);
    }
    class Withdrawal extends Base {
        blockNumber: number;
        to: string;
        nullifierHash: string;
        transactionHash: string;
        fee: string;
        constructor(obj: any);
    }
    class Note extends Base {
        network: string;
        token: string;
        denomination: string;
        note: string;
        constructor(index: number, network: string, token: string, denomination: string, note: string);
    }
    class Invoice extends Base {
        network: string;
        token: string;
        denomination: string;
        invoice: string;
        constructor(index: number, network: string, token: string, denomination: string, invoice: string);
    }
    class Relayer extends Base {
        address: string;
        version: string;
        serviceFeePercent: number;
        miningFeePercent: number;
        status: string;
        chainId: number;
        prices: Map<string, BigNumber>;
        constructor(url: string, properties: RelayerProperties);
    }
}
export declare namespace Cache {
    type Rows<T extends Docs.Base> = Array<{
        doc?: T;
        id: PouchDB.Core.DocumentId;
        key: PouchDB.Core.DocumentKey;
        value: {
            rev: PouchDB.Core.RevisionId;
            deleted?: boolean;
        };
    }>;
    type RowsOptions = PouchDB.Core.AllDocsWithKeyOptions | PouchDB.Core.AllDocsWithinRangeOptions | PouchDB.Core.AllDocsOptions;
    export class Base<T extends Docs.Base> {
        private _adapter;
        private _path;
        name: string;
        isOpen: boolean;
        db: PouchDB.Database<T>;
        private _unzip;
        constructor(name: string, options?: Options.Cache);
        zip(outDirPath?: string, close?: boolean): Promise<void>;
        jsonify(outDirPath?: string): Promise<void>;
        get(keys: Array<any>): Promise<T>;
        getRows(emptyError?: Error, options?: RowsOptions): Promise<Rows<T>>;
        close(): Promise<void>;
        clear(): Promise<void>;
    }
    export abstract class Syncable<T extends Docs.Base> extends Base<T> {
        pooler?: AsyncUtils.PromisePooler;
        constructor(name: string, options?: Options.Cache);
        abstract buildDoc(response: any): Docs.Base;
        abstract getCallbacks(...args: Array<any>): Array<AsyncUtils.Callback>;
        abstract getErrorHandlers(...args: Array<any>): Array<AsyncUtils.ErrorHandler>;
        initializePooler(callbacks: Array<AsyncUtils.Callback>, errorHandlers: Array<AsyncUtils.ErrorHandler>, concurrencyLimit: number): void;
        close(): Promise<void>;
        clear(): Promise<void>;
    }
    export {};
}
//# sourceMappingURL=index.d.ts.map