"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cache = exports.Docs = exports.Constants = exports.Offchain = exports.Onchain = exports.Json = exports.Files = void 0;
// Archiving and zipping
const tar_1 = __importDefault(require("tar"));
// Fs
const fs_1 = require("fs");
const promises_1 = require("fs/promises");
const readline_1 = require("readline");
// Local logic
const sdk_utils_1 = require("@tornado/sdk-utils");
// PouchDB
const pouchdb_1 = __importDefault(require("pouchdb"));
const PouchDBAdapterMemory = __importStar(require("pouchdb-adapter-memory"));
// @ts-ignore
const pouchdb_collate_1 = require("pouchdb-collate");
// Register plugins
pouchdb_1.default.plugin(PouchDBAdapterMemory);
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ REST ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
var Files;
(function (Files) {
    function parentPath(filepath) {
        let path = filepath.split('/').slice(0, -1).join('/');
        path = path[path.length - 1] === '/' ? path : path + '/';
        return path;
    }
    Files.parentPath = parentPath;
    function stripExtensions(filepath) {
        const tokens = filepath.split('/');
        const stripped = tokens[tokens.length - 1].split('.')[0];
        const prefix = tokens.slice(0, -1).join('/');
        return prefix + (prefix !== '' ? '/' : '') + stripped;
    }
    Files.stripExtensions = stripExtensions;
    Files.getModulesPath = (relative, prefix) => (prefix ?? __dirname + '/../../node_modules/') + (relative ?? '');
    Files.getResourcePath = (relative, prefix) => (prefix ?? __dirname + '/../resources/') + (relative ?? '');
    Files.getCachePath = (relative, prefix) => (prefix ?? process.cwd() + '/cache/') + (relative ?? '');
    Files.cacheDirExists = (prefix) => (0, fs_1.existsSync)(Files.getCachePath('', prefix));
    Files.makeCacheDir = (prefix) => (0, fs_1.mkdirSync)(Files.getCachePath('', prefix));
    Files.loadRaw = (relative) => (0, promises_1.readFile)(Files.getResourcePath(relative));
    Files.loadRawSync = (relative) => (0, fs_1.readFileSync)(Files.getResourcePath(relative));
    Files.writeRaw = (relative, data, flag) => (0, promises_1.writeFile)(Files.getCachePath('', relative), data, { flag: flag });
    Files.writeRawSync = (relative, data, flag) => (0, fs_1.writeFileSync)(Files.getCachePath('', relative), data, { flag: flag });
    function gzipSync(fileOrDirPath, archivePath) {
        try {
            const tokens = fileOrDirPath.split('/');
            tar_1.default.create({
                cwd: parentPath(fileOrDirPath),
                file: stripExtensions(archivePath) + '.tar.gz',
                gzip: true,
                sync: true
            }, [tokens[tokens.length - 1]]);
        }
        catch (err) {
            throw sdk_utils_1.ErrorUtils.ensureError(err);
        }
    }
    Files.gzipSync = gzipSync;
    function gunzipSync(archivePath, extractPath) {
        try {
            tar_1.default.extract({
                cwd: extractPath,
                file: stripExtensions(archivePath) + '.tar.gz',
                sync: true
            });
        }
        catch (err) {
            throw sdk_utils_1.ErrorUtils.ensureError(err);
        }
    }
    Files.gunzipSync = gunzipSync;
    async function wipeCache(prompt = true, dirPath) {
        const dir = await (0, promises_1.opendir)(Files.getCachePath('', dirPath));
        const toRemove = [];
        const userInput = (0, readline_1.createInterface)({ input: process.stdin, output: process.stdout });
        for await (const entry of dir) {
            if (entry.name.match('(Deposit.*)|(Withdrawal.*)|(Note.*)|(Invoice.*)'))
                toRemove.push(Files.getCachePath(entry.name, dirPath));
        }
        if (toRemove.length != 0) {
            if (prompt) {
                const promptString = `\nCache wipe requested, following would be wiped:\n\n${toRemove.join('\n')}\n\nContinue? (y/n): `;
                function wipeCachePrompt(prompt, resolve) {
                    userInput.question(prompt, (answer) => {
                        if (answer == 'y') {
                            userInput.close();
                            resolve(true);
                        }
                        else if (answer == 'n') {
                            userInput.close();
                            resolve(false);
                        }
                        else
                            wipeCachePrompt('', resolve);
                    });
                }
                const answer = await new Promise((resolve) => wipeCachePrompt(promptString, resolve));
                if (answer)
                    await Promise.all(toRemove.map((entry) => (0, promises_1.rm)(entry, { recursive: true, force: true }))).catch((err) => {
                        throw sdk_utils_1.ErrorUtils.ensureError(err);
                    });
            }
            else {
                await Promise.all(toRemove.map((entry) => (0, promises_1.rm)(entry, { recursive: true, force: true }))).catch((err) => {
                    throw sdk_utils_1.ErrorUtils.ensureError(err);
                });
            }
        }
    }
    Files.wipeCache = wipeCache;
})(Files = exports.Files || (exports.Files = {}));
var Json;
(function (Json) {
    const cachedJsonData = new Map();
    async function load(relativePath, encoding = 'utf8', pathGetter = Files.getResourcePath) {
        if (cachedJsonData.has(relativePath))
            return cachedJsonData.get(relativePath);
        else {
            const obj = JSON.parse(await (0, promises_1.readFile)(pathGetter(relativePath), encoding));
            cachedJsonData.set(relativePath, obj);
            return obj;
        }
    }
    Json.load = load;
    function loadSync(relativePath, encoding = 'utf8', pathGetter = Files.getResourcePath) {
        if (cachedJsonData.has(relativePath))
            return cachedJsonData.get(relativePath);
        else {
            const obj = JSON.parse((0, fs_1.readFileSync)(pathGetter(relativePath), encoding));
            cachedJsonData.set(relativePath, obj);
            return obj;
        }
    }
    Json.loadSync = loadSync;
    function toMap(jsonData) {
        return new Map(Object.entries(jsonData));
    }
    Json.toMap = toMap;
    async function loadMap(relativePath, encoding = 'utf8') {
        return toMap(await load(relativePath, encoding));
    }
    Json.loadMap = loadMap;
    function getError(...values) {
        return sdk_utils_1.ErrorUtils.getError(`there is no such entry for the key-value path [${values.join('][')}]`);
    }
    Json.getError = getError;
    function throwError(...values) {
        throw getError(...values);
    }
    Json.throwError = throwError;
    function getValue(jsonObj, keys) {
        for (let i = 0; i < keys.length; i++) {
            jsonObj = jsonObj[keys[i]] ?? throwError(...keys.slice(0, i + 1));
        }
        return jsonObj;
    }
    Json.getValue = getValue;
})(Json = exports.Json || (exports.Json = {}));
// TODO: Decide whether to also cache the data instead of just loading it for the function call
var Onchain;
(function (Onchain) {
    async function getClassicInstanceData(network, token, denomination) {
        const instanceData = Json.getValue(await Json.load('onchain/instances.json'), [network, token]);
        return {
            network: +network,
            symbol: token.toUpperCase(),
            decimals: Json.getValue(instanceData, ['decimals']),
            denomination: +denomination,
            deployBlock: Json.getValue(instanceData, ['deployedBlockNumber', denomination]),
            address: Json.getValue(instanceData, ['instanceAddress', denomination]),
            anonymityMiningEnabled: Json.getValue(instanceData, ['miningEnabled'])
        };
    }
    Onchain.getClassicInstanceData = getClassicInstanceData;
    function getClassicInstanceDataSync(network, token, denomination) {
        const instanceData = Json.getValue(Json.loadSync('onchain/instances.json'), [network, token]);
        return {
            network: +network,
            symbol: token.toUpperCase(),
            decimals: Json.getValue(instanceData, ['decimals']),
            denomination: +denomination,
            deployBlock: Json.getValue(instanceData, ['deployedBlockNumber', denomination]),
            address: Json.getValue(instanceData, ['instanceAddress', denomination]),
            anonymityMiningEnabled: Json.getValue(instanceData, ['miningEnabled'])
        };
    }
    Onchain.getClassicInstanceDataSync = getClassicInstanceDataSync;
    async function getInstanceLookupKeys(instanceAddress) {
        // lookup some stuff first
        const lookupObj = await Json.load('onchain/instanceAddresses.json');
        const pathstring = Object.entries(lookupObj).find((el) => el[1] === instanceAddress)[0];
        const network = pathstring.match('[0-9]+')[0], token = pathstring.substring(network.length).match('[a-z]+')[0], denomination = pathstring.substring(network.length + token.length);
        return {
            network: network,
            token: token,
            denomination: denomination
        };
    }
    Onchain.getInstanceLookupKeys = getInstanceLookupKeys;
    function getInstanceLookupKeysSync(instanceAddress) {
        // lookup some stuff first
        const lookupObj = Json.loadSync('onchain/instanceAddresses.json');
        const pathstring = Object.entries(lookupObj).find((el) => el[1] === instanceAddress)[0];
        const network = pathstring.match('[0-9]+')[0], token = pathstring.substring(network.length).match('[a-z]+')[0], denomination = pathstring.substring(network.length + token.length);
        return {
            network: network,
            token: token,
            denomination: denomination
        };
    }
    Onchain.getInstanceLookupKeysSync = getInstanceLookupKeysSync;
    async function getPathstringBasedContent(filepath, paths) {
        const obj = await Json.load(filepath);
        return await Promise.all(paths.map((path) => Json.getValue(obj, [`${path.network ?? ''}${path.token ?? ''}${path.denomination ?? ''}`])));
    }
    Onchain.getPathstringBasedContent = getPathstringBasedContent;
    function getPathstringBasedContentSync(filepath, paths) {
        return paths.map((path) => Json.getValue(Json.loadSync(filepath), [
            `${path.network ?? ''}${path.token ?? ''}${path.denomination ?? ''}`
        ]));
    }
    Onchain.getPathstringBasedContentSync = getPathstringBasedContentSync;
    async function getNetworkSymbol(networkId) {
        return (await getPathstringBasedContent('onchain/networkSymbols.json', [{ network: networkId }]))[0];
    }
    Onchain.getNetworkSymbol = getNetworkSymbol;
    function getNetworkSymbolSync(networkId) {
        return getPathstringBasedContentSync('onchain/networkSymbols.json', [{ network: networkId }])[0];
    }
    Onchain.getNetworkSymbolSync = getNetworkSymbolSync;
    function getInstanceAddresses(paths) {
        return getPathstringBasedContent('onchain/instanceAddresses.json', paths);
    }
    Onchain.getInstanceAddresses = getInstanceAddresses;
    function getInstanceAddressesSync(paths) {
        return getPathstringBasedContentSync('onchain/instanceAddresses.json', paths);
    }
    Onchain.getInstanceAddressesSync = getInstanceAddressesSync;
    async function getInstanceAddress(network, token, denomination) {
        return (await getInstanceAddresses([{ network: network, token: token, denomination: denomination }]))[0];
    }
    Onchain.getInstanceAddress = getInstanceAddress;
    function getInstanceAddressSync(network, token, denomination) {
        return getInstanceAddressesSync([{ network: network, token: token, denomination: denomination }])[0];
    }
    Onchain.getInstanceAddressSync = getInstanceAddressSync;
    async function getRegistryAddress(network) {
        network = network ?? '1';
        return Json.getValue(await Json.load('onchain/infrastructure.json'), [network, 'registry']);
    }
    Onchain.getRegistryAddress = getRegistryAddress;
    function getRegistryAddressSync(network) {
        network = network ?? '1';
        return Json.getValue(Json.loadSync('onchain/infrastructure.json'), [network, 'registry']);
    }
    Onchain.getRegistryAddressSync = getRegistryAddressSync;
    async function getRegistryDeployBlockNum(network) {
        network = network ?? '1';
        return Json.getValue(await Json.load('onchain/deployedBlockNumbers.json'), [`${network}registry`]);
    }
    Onchain.getRegistryDeployBlockNum = getRegistryDeployBlockNum;
    function getRegistryDeployBlockNumSync(network) {
        network = network ?? '1';
        return Json.getValue(Json.loadSync('onchain/deployedBlockNumbers.json'), [`${network}registry`]);
    }
    Onchain.getRegistryDeployBlockNumSync = getRegistryDeployBlockNumSync;
    function getInstanceDeployBlockNums(paths) {
        return getPathstringBasedContent('onchain/deployedBlockNumbers.json', paths);
    }
    Onchain.getInstanceDeployBlockNums = getInstanceDeployBlockNums;
    function getInstanceDeployBlockNumsSync(paths) {
        return getPathstringBasedContentSync('onchain/deployedBlockNumbers.json', paths);
    }
    Onchain.getInstanceDeployBlockNumsSync = getInstanceDeployBlockNumsSync;
    async function getInstanceDeployBlockNum(network, token, denomination) {
        return (await getInstanceDeployBlockNums([{ network: network, token: token, denomination: denomination }]))[0];
    }
    Onchain.getInstanceDeployBlockNum = getInstanceDeployBlockNum;
    function getInstanceDeployBlockNumSync(network, token, denomination) {
        return getInstanceDeployBlockNumsSync([{ network: network, token: token, denomination: denomination }])[0];
    }
    Onchain.getInstanceDeployBlockNumSync = getInstanceDeployBlockNumSync;
    async function getProxyAddress(network) {
        return Json.getValue(await Json.load('onchain/infrastructure.json'), [network, 'proxy']);
    }
    Onchain.getProxyAddress = getProxyAddress;
    async function getMulticallAddress(network) {
        return Json.getValue(await Json.load('onchain/infrastructure.json'), [network, 'multicall']);
    }
    Onchain.getMulticallAddress = getMulticallAddress;
    async function getMulticall3Address(network) {
        return Json.getValue(await Json.load('onchain/infrastructure.json'), [network, 'multicall3']);
    }
    Onchain.getMulticall3Address = getMulticall3Address;
    function getProxyAddressSync(network) {
        return Json.getValue(Json.loadSync('onchain/infrastructure.json'), [network, 'proxy']);
    }
    Onchain.getProxyAddressSync = getProxyAddressSync;
    function getMulticallAddressSync(network) {
        return Json.getValue(Json.loadSync('onchain/infrastructure.json'), [network, 'multicall']);
    }
    Onchain.getMulticallAddressSync = getMulticallAddressSync;
    function getMulticall3AddressSync(network) {
        return Json.getValue(Json.loadSync('onchain/infrastructure.json'), [network, 'multicall3']);
    }
    Onchain.getMulticall3AddressSync = getMulticall3AddressSync;
    async function getTokenAddress(network, token) {
        return (await getPathstringBasedContent('onchain/tokenAddresses.json', [
            { network: network, token: token }
        ]))[0];
    }
    Onchain.getTokenAddress = getTokenAddress;
    function getTokenAddressSync(network, token) {
        return getPathstringBasedContentSync('onchain/tokenAddresses.json', [
            { network: network, token: token }
        ])[0];
    }
    Onchain.getTokenAddressSync = getTokenAddressSync;
    async function getTokenDecimals(network, token) {
        return (await getPathstringBasedContent('onchain/decimals.json', [{ network: network, token: token }]))[0];
    }
    Onchain.getTokenDecimals = getTokenDecimals;
    function getTokenDecimalsSync(network, token) {
        return getPathstringBasedContentSync('onchain/decimals.json', [
            { network: network, token: token }
        ])[0];
    }
    Onchain.getTokenDecimalsSync = getTokenDecimalsSync;
})(Onchain = exports.Onchain || (exports.Onchain = {}));
var Offchain;
(function (Offchain) {
    async function getUncensoredRpcURL(network, name = '') {
        const rpcs = Json.toMap(Json.getValue(await Json.load('offchain/infrastructure.json'), ['jrpc-uncensored', network]));
        if (name.length !== 0) {
            return rpcs.get(name);
        }
        let keys = rpcs.keys();
        let randCount = sdk_utils_1.NumberUtils.getRandomFromRange(0, rpcs.size - 1);
        for (let i = 0; i < randCount; i++)
            keys.next();
        return rpcs.get(keys.next().value);
    }
    Offchain.getUncensoredRpcURL = getUncensoredRpcURL;
    function getUncensoredRpcURLSync(network, name = '') {
        const rpcs = Json.toMap(Json.getValue(Json.loadSync('offchain/infrastructure.json'), ['jrpc-uncensored', network]));
        if (name.length !== 0) {
            return rpcs.get(name);
        }
        let keys = rpcs.keys();
        let randCount = sdk_utils_1.NumberUtils.getRandomFromRange(0, rpcs.size - 1);
        for (let i = 0; i < randCount; i++)
            keys.next();
        return rpcs.get(keys.next().value);
    }
    Offchain.getUncensoredRpcURLSync = getUncensoredRpcURLSync;
    async function getClassicSubgraphURL(network) {
        return Json.getValue(await Json.load('offchain/infrastructure.json'), ['subgraph', network]);
    }
    Offchain.getClassicSubgraphURL = getClassicSubgraphURL;
    function getClassicSubgraphURLSync(network) {
        return Json.getValue(Json.loadSync('offchain/infrastructure.json'), ['subgraph', network]);
    }
    Offchain.getClassicSubgraphURLSync = getClassicSubgraphURLSync;
})(Offchain = exports.Offchain || (exports.Offchain = {}));
var Constants;
(function (Constants) {
    Constants.MERKLE_TREE_HEIGHT = 20;
})(Constants = exports.Constants || (exports.Constants = {}));
var Docs;
(function (Docs) {
    // TODO: Probably find some easier way to lookup below docs for the end user...
    class Base {
        _id;
        _rev;
        constructor(id) {
            this._id = id;
        }
    }
    Docs.Base = Base;
    class Deposit extends Base {
        blockNumber;
        leafIndex;
        commitment;
        transactionHash;
        timestamp;
        constructor(obj) {
            const blockNumber = obj['blockNumber'];
            const transactionHash = obj['transactionHash'];
            const commitment = obj['args']['commitment'];
            const leafIndex = obj['args']['leafIndex'];
            const timestamp = obj['args']['timestamp'].toString();
            // To preserve order because we will need it later
            super((0, pouchdb_collate_1.toIndexableString)([blockNumber, leafIndex, commitment]));
            this.commitment = commitment;
            this.blockNumber = blockNumber;
            this.leafIndex = leafIndex;
            this.transactionHash = transactionHash;
            this.timestamp = timestamp;
        }
    }
    Docs.Deposit = Deposit;
    class Withdrawal extends Base {
        blockNumber;
        to;
        nullifierHash;
        transactionHash;
        fee;
        constructor(obj) {
            const blockNumber = obj['blockNumber'];
            const transactionHash = obj['transactionHash'];
            const to = obj['args']['to'];
            const nullifierHash = obj['args']['nullifierHash'];
            const fee = obj['args']['fee'].toString();
            super((0, pouchdb_collate_1.toIndexableString)([blockNumber, to, nullifierHash]));
            this.blockNumber = blockNumber;
            this.to = to;
            this.nullifierHash = nullifierHash;
            this.transactionHash = transactionHash;
            this.fee = fee;
        }
    }
    Docs.Withdrawal = Withdrawal;
    class Note extends Base {
        network;
        token;
        denomination;
        note;
        constructor(index, network, token, denomination, note) {
            super((0, pouchdb_collate_1.toIndexableString)([index, network, denomination, token]));
            this.network = network;
            this.token = token;
            this.denomination = denomination;
            this.note = note;
        }
    }
    Docs.Note = Note;
    class Invoice extends Base {
        network;
        token;
        denomination;
        invoice;
        constructor(index, network, token, denomination, invoice) {
            super((0, pouchdb_collate_1.toIndexableString)([index, network, denomination, token]));
            this.network = network;
            this.token = token;
            this.denomination = denomination;
            this.invoice = invoice;
        }
    }
    Docs.Invoice = Invoice;
    class Relayer extends Base {
        address;
        version;
        serviceFeePercent;
        miningFeePercent;
        status;
        chainId;
        prices;
        constructor(url, properties) {
            super((0, pouchdb_collate_1.toIndexableString)([url]));
            this.address = properties.address;
            this.version = properties.version;
            this.serviceFeePercent = properties.serviceFeePercent;
            this.miningFeePercent = properties.miningFeePercent;
            this.status = properties.status;
            this.chainId = properties.chainId;
            this.prices = properties.prices;
        }
    }
    Docs.Relayer = Relayer;
})(Docs = exports.Docs || (exports.Docs = {}));
var Cache;
(function (Cache) {
    class Base {
        _adapter;
        _path;
        name;
        isOpen;
        db;
        _unzip(dirPath) {
            if ((0, fs_1.existsSync)(this._path + '.tar.gz')) {
                if ((0, fs_1.existsSync)(this._path)) {
                    throw sdk_utils_1.ErrorUtils.getError(`Can't load both ${this.name} and ${this.name + '.tar.gz'}, remove one!`);
                }
                else
                    Files.gunzipSync(this._path, Files.getCachePath('', dirPath));
            }
        }
        constructor(name, options) {
            this.name = name;
            if (options?.persistent === false && options?.adapter !== 'memory' && options?.adapter !== null)
                throw sdk_utils_1.ErrorUtils.getError('Cache.new: if not persistent, cache must use memory adapter.');
            if (options?.adapter === 'memory' && options?.persistent === true)
                throw sdk_utils_1.ErrorUtils.getError("Cache.new: can't specify memory adapter if persistent.");
            const dbAdapter = options?.adapter ?? (options?.persistent === false ? 'memory' : 'leveldb');
            if (options?.dirPath)
                if (options.dirPath.charAt(options.dirPath.length - 1) != '/')
                    options.dirPath += '/';
            if (!Files.cacheDirExists(options?.dirPath))
                Files.makeCacheDir();
            this._path = Files.getCachePath(name, options?.dirPath);
            this._adapter = dbAdapter;
            this._unzip(options?.dirPath);
            this.db = new pouchdb_1.default(this._path, { adapter: dbAdapter });
            this.isOpen = true;
        }
        async zip(outDirPath, close = false) {
            await this.close();
            if (outDirPath)
                outDirPath = outDirPath[outDirPath.length - 1] !== '/' ? outDirPath + '/' : outDirPath;
            Files.gzipSync(this._path, (outDirPath ?? Files.parentPath(this._path)) + this.name);
            if (!close)
                this.db = new pouchdb_1.default(this._path, { adapter: this._adapter });
            this.isOpen = !close;
        }
        async jsonify(outDirPath) {
            const docs = (await this.getRows()).map((row) => {
                row.doc._rev = undefined;
                return row.doc;
            });
            if (outDirPath)
                outDirPath = outDirPath[outDirPath.length - 1] !== '/' ? outDirPath + '/' : outDirPath;
            Files.writeRawSync((outDirPath ?? Files.parentPath(this._path)) + this.name + '.json', JSON.stringify(docs, null, 2), 'w');
        }
        async get(keys) {
            return await this.db.get((0, pouchdb_collate_1.toIndexableString)(keys)).catch((err) => {
                throw sdk_utils_1.ErrorUtils.ensureError(err);
            });
        }
        async getRows(emptyError = sdk_utils_1.ErrorUtils.getError(`Base.getRows: there is no cache entry for ${this.name}`), options = { include_docs: true, attachments: false }) {
            const docs = await this.db.allDocs(options).catch((err) => {
                throw sdk_utils_1.ErrorUtils.ensureError(err);
            });
            // If calling from an external function that wants this to fail
            // For example if the DB was opened in one invocation
            // Then fail and delete the EMTPY db
            if (docs.total_rows === 0 && emptyError) {
                await this.clear();
                throw emptyError;
            }
            return docs.rows;
        }
        async close() {
            await this.db.close().catch((err) => {
                throw sdk_utils_1.ErrorUtils.ensureError(err);
            });
        }
        async clear() {
            await this.db.destroy().catch((err) => {
                throw sdk_utils_1.ErrorUtils.ensureError(err);
            });
        }
    }
    Cache.Base = Base;
    class Syncable extends Base {
        pooler;
        constructor(name, options) {
            super(name, options);
        }
        initializePooler(callbacks, errorHandlers, concurrencyLimit) {
            if (this.pooler)
                this.pooler.reset();
            this.pooler = new sdk_utils_1.AsyncUtils.PromisePooler(callbacks, errorHandlers, concurrencyLimit);
        }
        async close() {
            if (this.pooler && this.pooler.pending)
                throw sdk_utils_1.ErrorUtils.getError("Syncable.close: can't clear while pooler still has pending promises.");
            await super.close();
        }
        async clear() {
            if (this.pooler && this.pooler.pending)
                throw sdk_utils_1.ErrorUtils.getError("Syncable.clear: can't clear while pooler still has pending promises.");
            await super.clear();
        }
    }
    Cache.Syncable = Syncable;
})(Cache = exports.Cache || (exports.Cache = {}));
//# sourceMappingURL=index.js.map